# Generated by Django 5.0.8 on 2024-10-02 07:34

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbcrm", "0010_alter_account_reference_id"),
    ]

    operations = [
        migrations.AddField(
            model_name="activity",
            name="summary",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AddField(
            model_name="activity",
            name="heat",
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]
